from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, RuntimeBusinessObject, Dataset
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpansionResponse(TcBaseObj):
    """
    'ExpansionResponse' contains a map with key as parent Fnd0BomLineLite or BOMLine and values as 'ChildLineInfo'.
    Additionally it also contains 'DatasetInfo' and uids of undelivered Fnd0BomLineLite objects.
    
    :var parentChildInfo: Map with key as parent Fnd0BomLineLite or BOMLine and values as' ChildLineInfo'. 
    :var datasetInfo: 'DatasetInfo' contains name of GRM relation through which underlying object of Fnd0BomLineLite or
    BOMLine is attached to Dataset, Dataset object and list of 'NamedRefInfo'.
    :var undeliveredLines: Fnd0BomLineLite uids which are yet to be transferred to client. 
    :var serviceData: The service data containing partial errors.
    """
    parentChildInfo: ParentLineToChildLineInfo = None
    datasetInfo: List[DatasetInfo] = ()
    undeliveredLines: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class LineAndPaths(TcBaseObj):
    """
    'LineAndPaths' contains beginning Fnd0BomLineLite/BOMLine and list of 'Paths' which needs to configured as
    Fnd0BomLineLite.
    
    :var startingLine: Fnd0BomLineLite or BOMLine as starting line from which child Fnd0BomLineLite to be constructed
    based on 'Paths'.
    :var paths: List of 'Path' where each element has attribute and value for which Fnd0BomLineLite to be constructed.
    """
    startingLine: RuntimeBusinessObject = None
    paths: List[Path] = ()


@dataclass
class NamedRefInfo(TcBaseObj):
    """
    Contains details of named reference object of Dataset. 
    
    :var name: Name of reference object in Dataset. For example, JTPart.
    :var type: The type of the named reference object.
    :var object: Object reference corresponding to the named reference.
    :var fileTicket: FMS ticket used to retrieve the file in cases where named reference objects to be returned. 
    """
    name: str = ''
    type: str = ''
    object: BusinessObject = None
    fileTicket: str = ''


@dataclass
class PartialExpansionControls(TcBaseObj):
    """
    Contains preferences to control how much to expand and return. Also describes 'RelationAndTypesCriteria' specifying
    relation name and Dataset information to query related objects.
    
    :var expansionPrefValues: Contains payload and Dataset retrieval preferences.
    Supported key, values and meaning are:
    
    Key: fetchRelatedObjects
    Specifies whether to return related Dataset objects as specified in relationCriteria. Valid values are 0 and 1.
    0 indicates do not return any related objects. This is the default value.
    1 indicates to return related objects as per relationCriteria.
    
    Key: minLinesWithRelatedObjects
    Specifies minimum lines to accumulate with related Dataset objects based on relationCriteria. This preference has
    significance only when fetchRelatedobjects is set 1.
    If not specified or specified as 0 related object check is not performed. Other valid value is positive integer.
    
    Key: outputPageSize
    Specifies number of child lines to return in response. When preference is not specified or passed as 0, all lines
    generated during operation are returned. Other valid value is positive integer.
    
    Key: wantSiblings
    Specifies whether to return siblings of Fnd0BomLineLite found based on 'Path'(s). 
    0 indicates to ignore sibling creation. This is default behavior.
    1 indicates return siblings Fnd0BomLineLite. Note that siblings are not expanded by this operation.
    
    Note that response is returned when either of minLinesWithRelatedObjects or outputPageSize criteria is satisfied.
    Siblings are also considered while calculating outputPageSize.
    
    
    
    
    
    
    
    :var relationCriteria: 'RelationAndTypesCriteria' contains GRM relation name, criteria to retrieve named reference
    objects and 'DatasetTypeAndNamedRefs'. This information is used to retrieve related Dataset, named reference
    objects and FMS tickets.
    """
    expansionPrefValues: StringKeyToIntValue = None
    relationCriteria: List[RelationTypesAndCriteria] = ()


@dataclass
class Path(TcBaseObj):
    """
    'Path' contains list of uids to be configured as Fnd0BomLineLite. The elements of the 'Path' must be in top to
    bottom order. The first value of elements must be associated to startingLine of 'LineAndPaths'. The last element of
    the path specifies end.
    
    :var pathId: Identifier of the path. 
    This is for future usage where client may request any one path with same id to be resolved.
    :var basedOn: List of string where string can be either AbsOccId, APN or CloneStableId. If length is not same as
    that of elements, last basedOn is assumed for remaining elements.
    :var elements: Contains list of AbsOccurrence puid, MEAppearancePathNode puid or clone stable occ uid of
    PSOccurrenceThread as identifier of PSOccurrence to configure as Fnd0BomLineLite.
    """
    pathId: str = ''
    basedOn: List[str] = ()
    elements: List[str] = ()


@dataclass
class RelationTypesAndCriteria(TcBaseObj):
    """
    Criteria to find Datasets and named reference objects associated to Fnd0BomLineLite objects.
    
    :var relationName: Name of GRM relation through Dataset is attached with underlying object of Fnd0BomLineLite or
    BOMLine. This field can not be empty.
    :var namedRefHandler: Describes criteria to retrieve named reference objects. 
    
    Default value is empty string, in such case named reference objects as specified in 'DatasetTypeAndNamedRefs' are
    retrieved.
    
    - No named reference objects are retrieved when specified as 'NoNamedRefs'. However, related Dataset as specified
    in attachmentTypeName of 'DatasetTypeAndNamedRefs' are retrieved. 
    - All named reference objects of Dataset to be retrieved when specified as 'AllNamedRefs'.  
    - Preferred JT will be returned when value is 'PreferredJT'.
    
    
    
    Note that namedReferencesNames of 'DatasetTypeAndNamedRefs' is ignored when value is 'NoNamedRefs', 'AllNamedRefs'
    or 'PreferredJT'.
    :var datasetTypeAndNamedRefs: A list of 'DatasetTypeAndNamedRefs'.
    """
    relationName: str = ''
    namedRefHandler: str = ''
    datasetTypeAndNamedRefs: List[DatasetTypeAndNamedRefs] = ()


@dataclass
class ChildLineInfo(TcBaseObj):
    """
    'ChildLineInfo' contains child line object, 'index to DatasetInfo' and configuration accuracy flag. 
    
    :var line: Child of BOMLine or Fnd0BomLineLite as key of 'ParentLineToChildLineInfo'. Child can be BOMLine or
    Fnd0BomLineLite.
    :var indexToAttachedObjInfos: Index to datasetInfos in 'ExpansionResponse'. 'DatasetInfos' at that index has
    information about  Dataset, named references objects etc for the line.
    :var configNotReliable: Indicates if configuration is not accurate. Set to true when there is Occurrence
    Effectivity or Incremental Change associated with PSOccurrence.
    """
    line: RuntimeBusinessObject = None
    indexToAttachedObjInfos: List[int] = ()
    configNotReliable: bool = False


@dataclass
class Controls(TcBaseObj):
    """
    'Controls' facilitates user to specify the payload and information to find related Dataset objects.
    
    :var expansionPref: Contains payload and Dataset retrieval preferences.
    Supported key, values and meaning are:
    
    Key: maxLevel
    Number of levels to be expanded for a expansion request. If not specified next immediate level is expanded. This is
    the default behavior.
    Other valid value is positive integer.
    
    Key: fetchRelatedObjects
    Specifies whether to return related Dataset objects as specified in relationCriteria. Valid values are 0 and 1.
    0 indicates do not return any related objects. This is the default value.
    1 indicates to return related objects as per relationCriteria.
    
    Key: minLinesWithRelatedObjects
    Specifies minimum lines to accumulate with related Dataset objects based on relationCriteria. This preference has
    significance only when fetchRelatedobjects is set 1.
    If not specified or specified as 0 related object check is not performed. Other valid value is positive integer.
    
    Key: outputPageSize
    Specifies number of child lines to return in response. When preference is not specified or passed as 0, all lines
    generated during operation are returned. Other valid value is any positive integer.
    
    Note that response is returned when either of maxLevel, minLinesWithRelatedObjects or outputPageSize criteria is
    satisfied.
    :var relationCriteria: 'RelationAndTypesCriteria' contains GRM relation name, criteria to retrieve named reference
    objects and 'DatasetTypeAndNamedRefs'. This information is used to retrieve related Dataset, named reference
    objects and FMS tickets.
    """
    expansionPref: StringKeyToIntValue = None
    relationCriteria: List[RelationTypesAndCriteria] = ()


@dataclass
class DatasetInfo(TcBaseObj):
    """
    'DatasetInfo' describes GRM relation name through which Dataset is attached with underlying object of
    Fnd0BomLineLite or BOMLine. It also contains 'NamedRefInfo' for Dataset requested through' DatasetTypeAndNamedRefs'.
    
    :var relationName: Name of GRM relation through which underlying object of Fnd0BomLineLite or BOMLine is related to
    Dataset object.
    :var dataset: Dataset attached to Fnd0BomLineLite or BOMLine.
    :var namedRefInfo: Named reference objects details for Dataset.
    """
    relationName: str = ''
    dataset: Dataset = None
    namedRefInfo: List[NamedRefInfo] = ()


@dataclass
class DatasetTypeAndNamedRefs(TcBaseObj):
    """
    Contains DatasetType name and named reference names for Dataset to be retrieved.
    
    :var datasetTypeName: Type of Dataset to be retrieved.
    :var namedReferenceNames: List of named references names for Dataset. For example, JTPart.
    """
    datasetTypeName: str = ''
    namedReferenceNames: List[str] = ()


"""
Map with key as Fnd0BomLineLite or BOMLine and 'ChildLineInfo' as values.
"""
ParentLineToChildLineInfo = Dict[RuntimeBusinessObject, List[ChildLineInfo]]


"""
Map with string as key and integer as value.
"""
StringKeyToIntValue = Dict[str, int]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportTemplateInput(TcBaseObj):
    """
    The ExportTemplateInput structure contains the flags that are required to filter the different type of export
    templates such as SpecTemplate, ObjectTemplate and ExcelTemplate from the database.
    
    :var getSpecTemplates: If set to true then it will query for SpecTemplates in database.
    :var getObjectTemplates: If set to true then it will query for ObjectTemplates in database.
    :var getExcelTemplates: If set to true then it will query for ExcelTemplates in database.
    :var getConfiguredSpecTemplate: This element is not used.
    :var getConfiguredObjectTemplate: This element is not used.
    """
    getSpecTemplates: bool = False
    getObjectTemplates: bool = False
    getExcelTemplates: bool = False
    getConfiguredSpecTemplate: bool = False
    getConfiguredObjectTemplate: bool = False


@dataclass
class ExportToApplicationInputData1(TcBaseObj):
    """
    The ExportToApplicationInputData1 structure represents all of the data necessary to export selected Teamcenter
    objects to MSWord/Excel.
    
    :var objectsToExport: The list of Teamcenter business objects to export.
    :var attributesToExport: The list of attributes to export.
    :var applicationFormat: The application format such as "'MSWordXML'", "'MSExcel'", "'MSExcelLive'",
    "'MSExcelReimport'", "'MSWordXMLLive'", "'MSExcelLiveBulkMode'".
    
    Supported application format for this operation
    
    - MSWordXML     This format is used for exporting Workspace objects to static MSWord application.
    - MSExcel    This format is used for exporting Teamcenter objects to static MSExcel  application.
    - MSExcelLive    This format is used for exporting Teamcenter objects to Live MSExcel  application.
    - MSExcelReimport    This format is used for exporting Teamcenter objects to MSExcel 
    
    
    application for reimport purpose.
    - MSExcelLiveBulkMode    This format is used for exporting Teamcenter objects to MSExcel  application for Bulk Live
    mode so that user can make all the property changes and save all the changes to Teamcenter on click of "Save to
    Teamcenter"button.
    - MSWordXMLLive    This format is used for exporting Workspace objects to Live MSWord application.
    
    
    :var templateId: The name of the Word/Excel template
    :var templateType: The type of the export template.
    :var objectTemplateOverride: The runtime object template override which is association of objectTemplate for a
    Business object type for a  chosen SpecTemplate.
    """
    objectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateId: str = ''
    templateType: str = ''
    objectTemplateOverride: List[TemplateOverride] = ()


@dataclass
class GetExportTemplateResponse(TcBaseObj):
    """
    The GetExportTemplateResponse Return represents the output of getExportTemplates operation which contains the
    information about the objects of type SpecTemplate, ObjectTemplate and ExcelTemplate that are queried from the
    database. Example- If the input filter flag (getSpecTemplates) is set to true in ExportTemplateInput structure,
    then the GetExportTemplateResponse structure will contain the all the objects of type SpecTemplate
    
    :var specTemplates: The list of objects of type SpecTemplate
    :var objectTemplats: The list of objects of type ObjectTemplate
    :var excelTemplates: The list of objects of type ExcelTemplate
    :var configuredSpecTemplates: This is not used
    :var configuredObjectTemplates: This is not used
    :var serviceData: The Service Data
    """
    specTemplates: List[str] = ()
    objectTemplats: List[str] = ()
    excelTemplates: List[str] = ()
    configuredSpecTemplates: List[ConfiguredTemplate] = ()
    configuredObjectTemplates: List[ConfiguredTemplate] = ()
    serviceData: ServiceData = None


@dataclass
class ImportFromApplicationInputData1(TcBaseObj):
    """
    The ImportFromApplicationInputData1 structure represents all of the data necessary to import a specification into
    Teamcenter or import templates to Teamcenter.
    
    :var transientFileWriteTicket: The file ticket of the. docx file to be imported into Teamcenter.
    :var applicationFormat: The application format suchas "'MSWordXML'", "'MSWordXMLLive'",
    "'MSWordXMLOverWriteCheck'", "'MSWordSpecTemplate'","'MSWordObjTemplate'","'MSExcelTemplate'" and
    "'MSWordSetContent'"
    
    Supported application formats for this operation
    
    - MSWordXML     This format is used for importing a MSWord document to Teamcenter.
    - MSWordXMLLive    This format is used for importing a Live MSWord document to Teamcenter.
    - MSWordXMLOverwriteCheck    This format is used for importing a  Live MSWord document to Teamcenter and check for
    overwrite condition on the object during setting of properties in database.
    - MSWordSetContent    This format is used for importing a  Live MSWord document to 
    
    
    Teamcenter. This format is used by the embedded viewer to set the rich text of SpecElement and to set the
    properties on the SpecElement.
    - MSWordSpecTemplate    This format is used for importing a SpecificationTemplate to 
    
    
    Teamcenter.
    - MSWordObjectTemplate    This format is used for importing a ObjectTemplate to Teamcenter.
    - MSWordExcelTemplate    This format is used for importing a ExcelTemplate to Teamcenter.
    
    
    :var createSpecElementType: The subtype of SpecElement to be created.
    :var specificationType: The subtype of Specification to be created.
    :var isLive: If the flag is set to true then it indicates "live" option.
    If the flag is set to false then it indicates "static" option.
    """
    transientFileWriteTicket: str = ''
    applicationFormat: str = ''
    createSpecElementType: str = ''
    specificationType: str = ''
    isLive: bool = False


@dataclass
class TemplateOverride(TcBaseObj):
    """
    The structure represents all of the data necessary to override object templates.
    
    :var businessObjectType: The type of business object.
    :var templateId: The name of object template.
    :var templateType: The type of object template.
    """
    businessObjectType: str = ''
    templateId: str = ''
    templateType: str = ''


@dataclass
class ConfiguredTemplate(TcBaseObj):
    """
    The 'ConfiguredTemplate' structure contains the information required to get IRDC configured export templates. BMIDE
    enables the user to configure templates based on the object type using the IRDC configuration.
    
    :var businessObjectType: Type name of business object.
    :var templateName: The name of the configured template on business object type.
    :var templateType: The type name of configured template.
    """
    businessObjectType: str = ''
    templateName: str = ''
    templateType: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2014_06.DataManagement import AutomaticMFGFeaturesAssignmentInputInfo, AutomaticMFGFeaturesAssignmentResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Manufacturing._2014_06.IPAManagement import GetDynamicIPALinesResponse, CleanDynamicIPALinesInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class IPAManagementService(TcService):

    @classmethod
    def cleanDynamicIPALines(cls, input: CleanDynamicIPALinesInfo) -> ServiceData:
        """
        The operation clears the dynamic IPA lines generated using SOA getDynamicIPALines for the input BOP lines.
        """
        return cls.execute_soa_method(
            method_name='cleanDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2014_06',
            service_name='IPAManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getDynamicIPALines(cls, inputBOPLines: List[BusinessObject]) -> GetDynamicIPALinesResponse:
        """
        This operation searches predecessors of the input BOP lines (i.e. one or more processes) in the process
        structure and returns aggregation of all incoming parts to these BOP lines. 
        The IPAs can be generated on the fly; for one or more processes in a process structure where the items are
        consumed from product structure.  The operation considers occurrence types defined in
        'MEDynamicIPAOccurrenceTypes' preference; to determine which occurrence types to aggregate from predecessors in
        the given process structure to generate the IPA. The default value for this preference will be 'MEConsumed'. 
        The dynamic IPA (DIPA) nodes created under the process will be added as 'MEDynamicWorkpiece' occurrence type.
        Map of input bop lines and their corresponding DIPA nodes will be returned.
        """
        return cls.execute_soa_method(
            method_name='getDynamicIPALines',
            library='Internal-Manufacturing',
            service_date='2014_06',
            service_name='IPAManagement',
            params={'inputBOPLines': inputBOPLines},
            response_cls=GetDynamicIPALinesResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def automaticMFGFeaturesAssignment(cls, input: AutomaticMFGFeaturesAssignmentInputInfo) -> AutomaticMFGFeaturesAssignmentResponse:
        """
        This operation automatically assigns the manufacturing feature (ArcWeld, WeldPoint) objects from the source
        structure to the given target structure.
        """
        return cls.execute_soa_method(
            method_name='automaticMFGFeaturesAssignment',
            library='Internal-Manufacturing',
            service_date='2014_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=AutomaticMFGFeaturesAssignmentResponse,
        )



from __future__ import annotations

from tcsoa.gen.GlobalMultiSite._2020_04.ImportExport import ImportObjectsFromPLMXMLWithDSMResp
from tcsoa.gen.BusinessObjects import TransferMode, ItemRevision
from typing import List
from tcsoa.gen.GlobalMultiSite._2007_12.ImportExport import NamesAndValues
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def importObjectsFromPLMXMLWithDSM(cls, xmlFileTicket: str, namedRefFolderPath: str, transfermode: TransferMode, icRev: ItemRevision, sessionOptions: List[NamesAndValues]) -> ImportObjectsFromPLMXMLWithDSMResp:
        """
        The operation imports the objects from a PLM XML file, and returns the .plmd file read tickets for the Dataset
        named reference files in the PLM XML file. The tickets are used to upload the files to FMS volume using Data
        Share Manager system. The PLM XML file must be uploaded to the transient volume before the calling of this
        operation.
        
        Use cases:
        Use Case 1: Import PLMXML file to database with Data Share Manager configured.
        User can import PLMXML file using the steps below:
        1)    Upload the XML file to the transient volume using the API getTransientFileTicketsForUpload from
        FileManagementService, API RegisterTickets and API uploadFilesToPlm from FmsFileCacheProxy.
        2)    Call the service operation importObjectsFromPLMXMLWithDSM by specifying the XML file ticket, the Dataset
        named reference files folder path, the export transfermode, the incremental change context, and session options
        as the input arguments.
        3)    Upload the Dataset named reference files to the FMS volume using the namedRefPLMDFileTickets in the
        returned ImportObjectsFromPLMXMLWithDSMResp as the input argument for API DownloadFilesWithDM from IFileManager.
        4)    Download the import log file using the logFileTicket in the returned ImportObjectsFromPLMXMLWithDSMResp
        and the API RegisterTicket and API DownLoadTransientFile from FmsFileCacheProxy.
        """
        return cls.execute_soa_method(
            method_name='importObjectsFromPLMXMLWithDSM',
            library='GlobalMultiSite',
            service_date='2020_04',
            service_name='ImportExport',
            params={'xmlFileTicket': xmlFileTicket, 'namedRefFolderPath': namedRefFolderPath, 'transfermode': transfermode, 'icRev': icRev, 'sessionOptions': sessionOptions},
            response_cls=ImportObjectsFromPLMXMLWithDSMResp,
        )



from __future__ import annotations

from tcsoa.gen.Workflow._2008_06.Workflow import AttachmentInfo, ProcessTemplates, ChangeStateInputInfo, AuditFile, ChangeStateOutput, InstanceInfo, Resources, Templates, SoaEPMSupportingValues, ContextData, GroupRoleRef, CreateSignoffs, AssignmentLists, AllOrAssigned, Tasks, GetResourcePoolOutput, SoaEPMAction, RemoveSignoffsInfo
from tcsoa.gen.BusinessObjects import BusinessObject, Signoff, EPMJob, WorkspaceObject, EPMAssignmentList, EPMTask
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def getResourcePool(cls, groupRoleRef: List[GroupRoleRef]) -> GetResourcePoolOutput:
        """
        getResourcePool - Returns regular or all member ResourcePools corresponding
        to the list of GroupRoleRef (group and role)
        """
        return cls.execute_soa_method(
            method_name='getResourcePool',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'groupRoleRef': groupRoleRef},
            response_cls=GetResourcePoolOutput,
        )

    @classmethod
    def getWorkflowTemplates(cls, targets: List[WorkspaceObject], allOrAssignedCriteria: AllOrAssigned) -> Templates:
        """
        Get the list of workflow templates given the list of target workspace objects and the All or Assigned criteria.
        
        Description
        This SOA will return all the ready to use and under construction templates if the allorAssined criteria is
        SOA_EPM_ALL.
        If the allOrAssignedCriteria is set to SOA_EPM_Assigned, this SOA will get the group information and the
        configured filtering criteria from the session
        and using the list of target objects, return the filtered list of workflow templates.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowTemplates',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'targets': targets, 'allOrAssignedCriteria': allOrAssignedCriteria},
            response_cls=Templates,
        )

    @classmethod
    def addAttachments(cls, task: EPMTask, attachments: AttachmentInfo) -> ServiceData:
        """
        addAttachments - Add attachments to a task.
        """
        return cls.execute_soa_method(
            method_name='addAttachments',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'task': task, 'attachments': attachments},
            response_cls=ServiceData,
        )

    @classmethod
    def listDefinitions(cls, name: str, templatestatus: int) -> ProcessTemplates:
        """
        Gets a list of process templates based on the criteria passed.
        
        Description
        if name and status are specified, all the templates matching the name and the status are returned.
        If only name is specified, only the available templates matching the name will be returned
        If only status is specified, all the templates with the specified status is returned.
        If both are not specified, all the available templates will be returned for a non-Dba
        """
        return cls.execute_soa_method(
            method_name='listDefinitions',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'name': name, 'templatestatus': templatestatus},
            response_cls=ProcessTemplates,
        )

    @classmethod
    def performAction(cls, task: EPMTask, action: SoaEPMAction, comments: str, password: str, supportingValue: SoaEPMSupportingValues, supportingObject: BusinessObject) -> ServiceData:
        """
        Performs workflow actions on a task.
        The actions that can be performed using this operation are :
        assign, start, complete, skip, suspend, resume, undo, perform, approve, reject, promote and demote.
        Note: Special case
        This operation can be used to set only comments without changing the decision for perform signoff task.
        To do that, use the 'EPM_no_action' as input for action argument and use the current decision for the
        supportingValue argument.
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='performAction',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'task': task, 'action': action, 'comments': comments, 'password': password, 'supportingValue': supportingValue, 'supportingObject': supportingObject},
            response_cls=ServiceData,
        )

    @classmethod
    def removeAttachments(cls, task: EPMTask, attachments: List[BusinessObject]) -> ServiceData:
        """
        removeAttachments - Remove attachments to a task.
        """
        return cls.execute_soa_method(
            method_name='removeAttachments',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'task': task, 'attachments': attachments},
            response_cls=ServiceData,
        )

    @classmethod
    def removeSignoffs(cls, signoffs: List[RemoveSignoffsInfo]) -> ServiceData:
        """
        addSignoffs - Add/remove and update signoffs on a task
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='removeSignoffs',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'signoffs': signoffs},
            response_cls=ServiceData,
        )

    @classmethod
    def viewAuditFile(cls, auditedObject: BusinessObject, isSignoffReport: bool) -> AuditFile:
        """
        viewAuditFile - get audit information on the selected object when stored in a file.
        By default, audit info is stored in a file in teamcenter.  This operation cannot be used to get audit
        info when audit manager is turned on.
        """
        return cls.execute_soa_method(
            method_name='viewAuditFile',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'auditedObject': auditedObject, 'isSignoffReport': isSignoffReport},
            response_cls=AuditFile,
        )

    @classmethod
    def createInstance(cls, startImmediately: bool, observerKey: str, name: str, subject: str, description: str, contextData: ContextData) -> InstanceInfo:
        """
        Workflow processes are the instantiations from workflow templates. This operation can be used to create a
        workflow process or sub process at a local or a remote site from a given workflow template. 
        """
        return cls.execute_soa_method(
            method_name='createInstance',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'startImmediately': startImmediately, 'observerKey': observerKey, 'name': name, 'subject': subject, 'description': description, 'contextData': contextData},
            response_cls=InstanceInfo,
        )

    @classmethod
    def addSignoffs(cls, signoffs: List[CreateSignoffs]) -> ServiceData:
        """
        addSignoffs : Add signoffs on a task
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='addSignoffs',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'signoffs': signoffs},
            response_cls=ServiceData,
        )

    @classmethod
    def delegateSignoff(cls, delegatee: BusinessObject, signoff: Signoff) -> ServiceData:
        """
        Delegate a signoff to a different groupmember or a resourcepool.
        """
        return cls.execute_soa_method(
            method_name='delegateSignoff',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'delegatee': delegatee, 'signoff': signoff},
            response_cls=ServiceData,
        )

    @classmethod
    def assignAllTasks(cls, process: EPMJob, assignmentList: EPMAssignmentList, resources: List[Resources]) -> ServiceData:
        """
        Assign a process assignment list to a process.
        Description:
        If the assignment list is given, it will use it to apply the assignment list to the
        process.  If the assignment is not given, it will loop through the Resources structure and create the list
        of task templates and list of resources, profiles and other information to apply the resources to
        the process. Thus at a given time either assignmentList and resources both cannot be null. either one of them
        can be null
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='assignAllTasks',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'process': process, 'assignmentList': assignmentList, 'resources': resources},
            response_cls=ServiceData,
        )

    @classmethod
    def getAllTasks(cls, process: EPMJob, state: int) -> Tasks:
        """
        Gets all the tasks in a process
        """
        return cls.execute_soa_method(
            method_name='getAllTasks',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'process': process, 'state': state},
            response_cls=Tasks,
        )

    @classmethod
    def getAssignmentLists(cls, names: List[str]) -> AssignmentLists:
        """
        Gets the process assignment list given the process assignment list names
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='getAssignmentLists',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'names': names},
            response_cls=AssignmentLists,
        )

    @classmethod
    def changeState(cls, stateInput: ChangeStateInputInfo) -> ChangeStateOutput:
        """
        changeState - Change the state of a process or a task at a remote site.
        """
        return cls.execute_soa_method(
            method_name='changeState',
            library='Workflow',
            service_date='2008_06',
            service_name='Workflow',
            params={'stateInput': stateInput},
            response_cls=ChangeStateOutput,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ListOfValues, CadAttrMappingDefinition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAllAttrMappingsResponse(TcBaseObj):
    """
    The response from the 'getAllAttrMappings2' operation.
    
    :var attrMappingInfos: A list of attribute mapping information.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' plain objects with
    'CadAttrMappingDefinition' objects and property descriptor LOV objects.
    """
    attrMappingInfos: List[AttrMappingInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AttrMappingInfo(TcBaseObj):
    """
    Attribute mapping information.
    
    :var cadAttrMappingDefinition: The CadAttrMappingDefinition object reference representing the mapping definition.
    :var propDesc: The property descriptor structure containing property information for the mapping definition
    property.
    """
    cadAttrMappingDefinition: CadAttrMappingDefinition = None
    propDesc: PropDescriptor = None


@dataclass
class PropDescriptor(TcBaseObj):
    """
    The 'PropDescriptor' struct describes information about the Teamcenter property
    
    :var propName: Name of the property
    :var displayName: Display name of the property. This is the localized name for the property.
    :var isModifiable: Specifies whether the property is modifiable
    :var attachedSpecifier: Attached specifier holds the following information: 
    - Is it a hierarchical LOV attachment? If it is hierarchical LOV attachment, the specifier value is 0;  This value
    can be 0 in following cases:
    -     Standard LOV attachment (Combobox, Range etc)
    -     Value Ddescription attachment only ( specifier value is 0 for the value attachment )
    -     To check whether it is hierarchical or not, one needs to verify the children LOVs (value filters from LOV).
    If there are children LOVs then it could be a hierarchical LOV.
    -  Is it attached with interdependent?
    - bool isInterdependent = ( specifier & (1 << 1) ) != 0;
    -  Is it attached with description attachment?
    - bool  isDescriptionAttach = ( specifier & (1 << 1) )!= 0;
    
    
    If attached with interdependent or description attachment what is the order of the attachment?
    int order = specifier >> 8;
    :var maxLength: Maximum length for a string property.
    :var lovAttachmentsCategory: Defines categories of LOVs attached to the property.
        0:  No attachments
        1:  Only isTrue condition is attached
        2:  Only session based conditions are attached
        3:  One or more object based conditions are attached
    
    :var interdependentProps: Interdependent properties for interdependent LOVs.
    :var defaultValue: Default value for the property. This is also sometimes known as the initial value of the property
    :var propValueType: Value type for the property in integer form: 
    PROP_untyped (0) No value
    PROP_char (1) Value is a single character
    PROP_date (2) Value is a date
    PROP_double (3) Value is a double
    PROP_float (4) Value is a float
    PROP_int (5) Value is an integer
    PROP_logical (6) Value is a logical
    PROP_short (7) Value is a short
    PROP_string (8) Value is a character string
    PROP_typed_reference (9) Value is a typed reference
    PROP_untyped_reference (10) Value is an untyped reference
    PROP_external_reference (11) Value is an external reference
    PROP_note (12) Value is a note
    PROP_typed_relation (13) Value is a typed relation
    PROP_untyped_relation (14) Value is an untyped relation
    :var propType: Type for the property in integer form: 
    PROP_unknown (0) Property type is unknown
    PROP_attribute (1)  Based on a POM attribute (int, string, ...)
    PROP_reference (2)  Based on a POM reference
    PROP_relation (3) Based on an ImanRelation
    PROP_compound (4) Based on a property from another type
    PROP_runtime (5) Based on a computed value
    :var isDisplayable: Boolean property indicating if the property should be displayed (true) or not (false).
    :var isArray: Specifies whether the property is an array or single value
    :var lov: ListOfValues object attached to the property (if any)
    :var isRequired: Specifies whether the property is required
    :var isEnabled: Specifies whether the property is enabled
    """
    propName: str = ''
    displayName: str = ''
    isModifiable: bool = False
    attachedSpecifier: int = 0
    maxLength: int = 0
    lovAttachmentsCategory: int = 0
    interdependentProps: List[str] = ()
    defaultValue: str = ''
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    lov: ListOfValues = None
    isRequired: bool = False
    isEnabled: bool = False
